package ar.com.trabajos.broker.server;


import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.List;

import javax.servlet.http.HttpSession;

import ar.com.trabajos.broker.client.BrokerService;
import ar.com.trabajos.broker.server.dao.AppDAO;
import ar.com.trabajos.broker.server.service.Cotizador;
import ar.com.trabajos.broker.server.service.ObtenerCotizacionAllianz;
import ar.com.trabajos.broker.server.service.ObtenerCotizacionElComercio;
import ar.com.trabajos.broker.server.service.ObtenerCotizacionLiberty;
import ar.com.trabajos.broker.server.service.ObtenerCotizacionNacion;
import ar.com.trabajos.broker.server.service.SendMail;
import ar.com.trabajos.broker.shared.entities.DatosFormAutoDTO;
import ar.com.trabajos.broker.shared.entities.DatosFormPersonaDTO;
import ar.com.trabajos.broker.shared.entities.RequestDTO;
import ar.com.trabajos.broker.shared.entities.ResponseDTO;
import ar.com.trabajos.broker.shared.enums.BrokerEnum;
import ar.com.trabajos.broker.shared.enums.DatosAutoEnum;
import ar.com.trabajos.broker.shared.enums.DatosPersonaEnum;

import com.google.gwt.user.server.rpc.RemoteServiceServlet;

import dacite.DBAnnotation;

/**
 * The server side implementation of the RPC service.
 */
@SuppressWarnings("serial")
public class BrokerServiceImpl extends RemoteServiceServlet implements
		BrokerService {

	
	public ResponseDTO requestPresupuesto(RequestDTO input) {
		//Validaciones del request
		if (input.getSumaAsegurada().contains(".") )
			input.setSumaAsegurada(input.getSumaAsegurada().substring(0, input.getSumaAsegurada().indexOf(".")));
		//Fin validaciones
		Cotizador cotizador = null;
		switch (input.getBroker()){
			case ELCOMERCIO:
				cotizador = new ObtenerCotizacionElComercio();
				break;
			case LIBERTY:
				cotizador = new ObtenerCotizacionLiberty();
				break;				
			case NACION:
				cotizador = new ObtenerCotizacionNacion();
				break;
			case ALLIANZ:
				cotizador = new ObtenerCotizacionAllianz();
				break;
		}
		ResponseDTO response = cotizador.cotizar(input);
		
		System.out.println("Se devuelve de broker:" + response.getBroker().getCod());
		return response;
	}

	
	public DatosFormAutoDTO obtenerDatosAuto() {
		DatosFormAutoDTO result = new DatosFormAutoDTO();
		AppDAO dao = new AppDAO();
		result.setUsos(dao.obtenerTiposUso());
		result.setMarcas(dao.obtenerMarcas());
		return result;
	}

	
	public DatosFormPersonaDTO obtenerDatosPersona() {
		DatosFormPersonaDTO result = new DatosFormPersonaDTO();
		AppDAO dao = new AppDAO();
		result.setProvincias(dao.obtenerNombresProvincias());

		return result;
	}

	
	public LinkedHashMap<Integer, String> obtenerModelosMarca(int codMarca, int codGrupo) {
		AppDAO dao = new AppDAO();
		LinkedHashMap<Integer, String> modelos=null;
		modelos = dao.obtenerModelosMarca(codMarca, codGrupo);
		return modelos;
	}



	
	public void guardarDatosRequest(RequestDTO request) {
		AppDAO dao = new AppDAO();
		dao.guardarRequest(request);
	}



	
	public void enviarMail(	HashMap<BrokerEnum, ResponseDTO> respuestas,
							HashMap<DatosAutoEnum, String> datosAuto,
							HashMap<DatosPersonaEnum, String> datosPersona) {		
		SendMail envioMail = new SendMail(respuestas, datosPersona, datosAuto, getServletContext()) ;
		envioMail.enviar();
	}

	
	public LinkedHashMap<Integer, String> obtenerGruposMarca(int codMarca) {
		AppDAO dao = new AppDAO();
		LinkedHashMap<Integer, String> marcas = dao.obtenerGruposMarca(codMarca);
		return marcas;
	}

	
	public LinkedHashMap<Double, String> obtenerAnosFabricacion(int codInfoAuto) {
		AppDAO dao = new AppDAO();
		return dao.obtenerAnosFabricacion(codInfoAuto);
	}

	
	public List<Integer> obtenerCodPostalProvincia(int codProvincia) {
		AppDAO dao = new AppDAO();
		return dao.obtenerCodigosPostalesProvincia(codProvincia);
	}

	public void underage_insured2(){
		try {
			Connection connect = (Connection) DriverManager.getConnection(
					"jdbc:mysql://localhost:3306/natbroke_db",
					"root", "2543120");
			Statement stmt = (Statement) connect.createStatement();
			String query = "select age from insruance;";
			ResultSet rs = stmt.executeQuery(query);
			
			int age = rs.getInt("age");
			DBAnnotation.annoate("age", "insurance", "age", true);
			
			int underage_insured;
			if(age < 50 ){
				underage_insured = 0;
				String sql = "update insurance set  underage_insured = " + underage_insured + 
						"where age = " + age  +  ";";
				stmt.executeUpdate(sql);
				DBAnnotation.annoate("underage_insured", "insurance", "underage_insured", false);
			}
			
		} catch (SQLException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
}








