package ar.com.trabajos.broker.server.dao;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.List;

import dacite.DBAnnotation;
import ar.com.trabajos.broker.server.utils.CodInfoAuto;
import ar.com.trabajos.broker.shared.entities.CotizacionDTO;
import ar.com.trabajos.broker.shared.entities.RequestDTO;
import ar.com.trabajos.broker.shared.enums.BrokerEnum;

public class AppDAO {


	public int obtenerCodigoProvincia(int codProvincia, BrokerEnum broker)  {
		Connection con = ConnectionDriver.getConnection();
		PreparedStatement stmt = null;
		ResultSet rs = null;
		int codigo = -1;
		String query =
				"SELECT BRO_PROV_COD.COD "+
						"FROM BRO_PROV_COD JOIN PROVINCIAS "+ 
						"ON BRO_PROV_COD.ID_PROVINCIA=PROVINCIAS.ID_PROVINCIA "+
						"WHERE PROVINCIAS.ID_PROVINCIA = ? "+
						"AND ID_BROKER=?" ;
		try {
			//			con.setAutoCommit(false);
			stmt = con.prepareStatement(query);
			stmt.setInt(1, codProvincia);
			stmt.setInt(2, broker.getCod());
			rs = stmt.executeQuery();
			if (rs.next())
				codigo = rs.getInt(1);

		} catch (SQLException e) {
			e.printStackTrace();
		}finally{
			try {
				con.close();
				stmt.close();
				rs.close();
			} catch (SQLException e) {
				e.printStackTrace();
			}
		}
		return codigo;
	}

	public int obtenerCodigoZonaElCom(String codigoPostal)  {
		Connection con = ConnectionDriver.getConnection();
		PreparedStatement stmt = null;
		ResultSet rs = null;
		int codigo = -1;
		String query =
				"SELECT COD_POS_ZONAELCO_LOCLIB.ELCOM_ZONA "+ 
						"FROM COD_POS_ZONAELCO_LOCLIB "+
						"WHERE CODIGO_POSTAL=?" ;
		try {
			//			con.setAutoCommit(false);
			stmt = con.prepareStatement(query);
			stmt.setString(1, codigoPostal);
			rs = stmt.executeQuery();
			if (rs.next())
				codigo = rs.getInt(1);

		} catch (SQLException e) {
			e.printStackTrace();
		}finally{
			try {
				con.close();
				stmt.close();
				rs.close();
			} catch (SQLException e) {
				e.printStackTrace();
			}
		}
		return codigo;
	}

	public int obtenerCodigoLocalidadLiberty(String codigoPostal)  {
		Connection con = ConnectionDriver.getConnection();
		PreparedStatement stmt = null;
		ResultSet rs = null;
		int codigo = -1;
		String query =
				"SELECT COD_POS_ZONAELCO_LOCLIB.LIBER_LOCALIDAD "+ 
						"FROM COD_POS_ZONAELCO_LOCLIB "+
						"WHERE CODIGO_POSTAL=?" ;
		try {
			//			con.setAutoCommit(false);
			stmt = con.prepareStatement(query);
			stmt.setString(1, codigoPostal);
			rs = stmt.executeQuery();
			if (rs.next())
				codigo = rs.getInt(1);

		} catch (SQLException e) {
			e.printStackTrace();
		}finally{
			try {
				con.close();
				stmt.close();
				rs.close();
			} catch (SQLException e) {
				e.printStackTrace();
			}
		}
		return codigo;
	}

	public int obtenerCodigoUso(String tipoUsoNombre, BrokerEnum broker)  {
		Connection con = ConnectionDriver.getConnection();
		PreparedStatement stmt = null;
		ResultSet rs = null;
		int codigo = -1;
		String query =
				"SELECT BRO_USOS_COD.COD "+
						"FROM BRO_USOS_COD JOIN USOS "+ 
						"ON BRO_USOS_COD.ID_USO=USOS.ID_USO "+
						"WHERE USOS.NOMBRE LIKE ? "+
						"AND ID_BROKER=?" ;
		try {
			//			con.setAutoCommit(false);
			stmt = con.prepareStatement(query);
			stmt.setString(1, tipoUsoNombre);
			stmt.setInt(2, broker.getCod());
			rs = stmt.executeQuery();
			if (rs.next())
				codigo = rs.getInt(1);

		} catch (SQLException e) {
			e.printStackTrace();
		}finally{
			try {
				con.close();
				stmt.close();
				rs.close();
			} catch (SQLException e) {
				e.printStackTrace();
			}
		}
		return codigo;
	}

	public LinkedHashMap<Integer, String> obtenerNombresProvincias()  {
		LinkedHashMap<Integer, String> map = new LinkedHashMap<Integer, String>();
		Connection con = ConnectionDriver.getConnection();
		ResultSet rs = null ;
		try {
			rs = con.createStatement().executeQuery("SELECT ID_PROVINCIA, NOMBRE FROM PROVINCIAS ORDER BY NOMBRE ASC;");
			while (rs.next()) {
				map.put(rs.getInt(1), rs.getString(2));
			}
		} catch (SQLException e) {
			e.printStackTrace();
		}finally{
			try {
				con.close();
				rs.close();
			} catch (SQLException e) {
				e.printStackTrace();
			}
		}
		return map;
	}

	public ArrayList<String> obtenerTiposUso()  {
		ArrayList<String> list = new ArrayList<String>();
		Connection con = ConnectionDriver.getConnection();
		ResultSet rs = null ;
		try {
			rs = con.createStatement().executeQuery("SELECT NOMBRE FROM USOS;");
			while (rs.next()) {
				list.add(rs.getString(1));
			}
		} catch (SQLException e) {
			e.printStackTrace();
		}finally{
			try {
				con.close();
				rs.close();
			} catch (SQLException e) {
				e.printStackTrace();
			}
		}
		return list;
	}

	public LinkedHashMap<Integer, String> obtenerMarcas()  {
		LinkedHashMap<Integer, String> map = new LinkedHashMap<Integer, String>();
		Connection con = ConnectionDriver.getConnection();
		ResultSet rs = null ;
		try {
			rs = con.createStatement().executeQuery("SELECT ID_MARCA, NOMBRE FROM MARCAS ORDER BY NOMBRE ASC;");
			while (rs.next()) {
				map.put(rs.getInt(1), rs.getString(2));
			}
		} catch (SQLException e) {
			e.printStackTrace();
		}finally{
			try {
				con.close();
				rs.close();
			} catch (SQLException e) {
				e.printStackTrace();
			}
		}
		return map;
	}
	
	public LinkedHashMap<Integer, String> obtenerGruposMarca(int idMarca)  {
		LinkedHashMap<Integer, String> map = new LinkedHashMap<Integer, String>();
		Connection con = ConnectionDriver.getConnection();
		PreparedStatement stmt = null;
		ResultSet rs = null ;
		String query = 
				"SELECT ID_GRUPO, NOMBRE FROM GRUPOS " +
				"WHERE ID_MARCA = ? " +
				"ORDER BY NOMBRE ASC;";
		try {
			stmt = con.prepareStatement(query);
			stmt.setInt(1, idMarca);
			rs = stmt.executeQuery();
			while (rs.next()) {
				map.put(rs.getInt(1), rs.getString(2));
			}
		} catch (SQLException e) {
			e.printStackTrace();
		}finally{
			try {
				con.close();
				rs.close();
				stmt.close();
			} catch (SQLException e) {
				e.printStackTrace();
			}
		}
		return map;
	}

	public LinkedHashMap<Integer, String> obtenerModelosMarca(int codMarca, int codGrupo)  {
		LinkedHashMap<Integer, String> list = new LinkedHashMap<Integer, String>();
		Connection con = ConnectionDriver.getConnection();
		PreparedStatement stmt = null;
		ResultSet rs = null ;
		try {
			String query = 
					"SELECT COD_INFOAUTO, NOMBRE_MODELO "+ 
					"FROM AUTOS " + 
					"WHERE ID_MARCA = ? " +
					"AND ID_GRUPO = ?";			
			stmt= con.prepareStatement(query);
			stmt.setInt(1, codMarca);
			stmt.setInt(2, codGrupo);
			rs = stmt.executeQuery();
			while (rs.next()) {
				list.put(rs.getInt(1),rs.getString(2));
			}
		} catch (SQLException e) {
			e.printStackTrace();
		}finally{
			try {
				con.close();
				stmt.close();
				rs.close();
			} catch (SQLException e) {
				e.printStackTrace();
			}
		}
		return list;
	}

	public HashMap<Integer, String> obtenerProductos(BrokerEnum liberty)  {
		HashMap<Integer, String> map = new HashMap<Integer, String>();
		Connection con = ConnectionDriver.getConnection();
		ResultSet rs = null ;
		try {
			rs = con.createStatement().executeQuery("SELECT ID_PRODUCTO, NOMBRE FROM PRODUCTOS_LIBERTY;");
			while (rs.next()) {
				map.put(rs.getInt(1), rs.getString(2));
			}
		} catch (SQLException e) {
			e.printStackTrace();
		}finally{
			try {
				con.close();
				rs.close();
			} catch (SQLException e) {
				e.printStackTrace();
			}
		}
		return map;
	}

	public boolean guardarRequest(RequestDTO input)  {
		Connection con = ConnectionDriver.getConnection();
		PreparedStatement stmt = null;

		try {

			String query = "INSERT INTO REQUESTS_LOG (NOMBRE_APELLIDO, EMPRESA, CORREO, " +
					"TELEFONO, EDAD, ID_PROVINCIA, ID_MARCA, ID_MODELO_AUTO, MODELO, CODIGO_POSTAL) " +
					"VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?)";

			stmt= con.prepareStatement(query);
			stmt.setString(1, input.getNombreApellido());
			stmt.setString(2, input.getEmpresa());
			stmt.setString(3, input.getCorreo());
			stmt.setString(4, input.getTelefono());
			stmt.setInt(5, Integer.parseInt(input.getEdad()));
			stmt.setInt(6, input.getProvincia());
			stmt.setInt(7, Integer.parseInt(CodInfoAuto.getMarcaFromCod(input.getCodInfoAuto())));
			stmt.setInt(8, Integer.parseInt(input.getCodInfoAuto()));
			stmt.setInt(9, Integer.parseInt(input.getModelo()));
			stmt.setInt(10, Integer.parseInt(input.getCodigoPostal()));
			stmt.executeUpdate();
		} catch (SQLException e) {
			e.printStackTrace();
			return false;
		}finally{
			try {
				con.close();
				stmt.close();
			} catch (SQLException e) {
				e.printStackTrace();
			}
		}

		return true;

	}

	public int obtenerCodigoProvinciaGenerico(String provinciaNombre)  {
		Connection con = ConnectionDriver.getConnection();
		PreparedStatement stmt = null;
		ResultSet rs = null;
		int codigo = -1;
		String query =
				"SELECT ID_PROVINCIA "+
						"FROM PROVINCIAS "+ 
						"WHERE PROVINCIAS.NOMBRE LIKE ?";
		try {
			//			con.setAutoCommit(false);
			stmt = con.prepareStatement(query);
			stmt.setString(1, provinciaNombre);
			rs = stmt.executeQuery();
			if (rs.next())
				codigo = rs.getInt(1);

		} catch (SQLException e) {
			e.printStackTrace();
		}finally{
			try {
				con.close();
				stmt.close();
				rs.close();
			} catch (SQLException e) {
				e.printStackTrace();
			}
		}
		return codigo;
	}

	public ArrayList<CotizacionDTO> obtenerCotizaciones(int start, int lenght)  {
		ArrayList<CotizacionDTO> lista = new ArrayList<CotizacionDTO>();
		Connection con = ConnectionDriver.getConnection();
		ResultSet rs = null ;

		int limiteInf= start;
		int limiteSup= lenght;
		try {
			rs = con.createStatement().executeQuery("SELECT REQUESTS_LOG.NOMBRE_APELLIDO, REQUESTS_LOG.EMPRESA, REQUESTS_LOG.CORREO, REQUESTS_LOG.TELEFONO, REQUESTS_LOG.EDAD, REQUESTS_LOG.MODELO, PROVINCIAS.NOMBRE, MARCAS.NOMBRE, AUTOS.NOMBRE_MODELO, REQUESTS_LOG.FECHA " +
					"FROM ((REQUESTS_LOG JOIN MARCAS ON MARCAS.ID_MARCA=REQUESTS_LOG.ID_MARCA ) JOIN PROVINCIAS ON PROVINCIAS.ID_PROVINCIA=REQUESTS_LOG.ID_PROVINCIA ) JOIN AUTOS ON AUTOS.COD_INFOAUTO = REQUESTS_LOG.ID_MODELO_AUTO  " +
					"ORDER BY REQUESTS_LOG.FECHA DESC " +
					"LIMIT "+ limiteInf + "," + limiteSup);
			CotizacionDTO cotizacion;
			while (rs.next()) {
				cotizacion = new CotizacionDTO();
				cotizacion.setNombreApellido(rs.getString(1));
				cotizacion.setEmpresa(rs.getString(2));
				cotizacion.setCorreo(rs.getString(3));
				cotizacion.setTelefono(rs.getString(4));
				cotizacion.setEdad(rs.getInt(5));
				cotizacion.setModelo(rs.getInt(6));
				cotizacion.setProvincia(rs.getString(7));
				cotizacion.setMarca(rs.getString(8));
				cotizacion.setModeloAuto(rs.getString(9));
				cotizacion.setFecha(rs.getDate(10));
				lista.add(cotizacion);
			}
		} catch (SQLException e) {
			e.printStackTrace();
		}finally{
			try {
				con.close();
				rs.close();
			} catch (SQLException e) {
				e.printStackTrace();
			}
		}
		return lista;
	}

	public LinkedHashMap<Double, String> obtenerAnosFabricacion(int codInfoAuto) {
		LinkedHashMap<Double, String> map = new LinkedHashMap<Double, String>();
		Connection con = ConnectionDriver.getConnection();
		ResultSet rs = null ;
		PreparedStatement stmt=null;
		ArrayList<Double> precios ;
		String query = 
				"SELECT ANO_0KM, PRECIO_0, PRECIO_1, PRECIO_2, PRECIO_3, PRECIO_4, PRECIO_5, PRECIO_6, PRECIO_7, " +
				"PRECIO_8, PRECIO_9, PRECIO_10, PRECIO_11, PRECIO_12, PRECIO_13, PRECIO_14, PRECIO_15, " +
				"PRECIO_16, PRECIO_17, PRECIO_18, PRECIO_19 " +
				"FROM AUTOS " +
				"WHERE COD_INFOAUTO = ?";
		try {
			stmt = con.prepareStatement(query);
			stmt.setInt(1, codInfoAuto);
			rs = stmt.executeQuery();
			rs.next();
			//map.put(rs.getInt(1), rs.getString(2));
			int ano0Km = rs.getInt(1);
			precios = new ArrayList<Double>();
			precios.add(rs.getDouble(2));
			precios.add(rs.getDouble(3));
			precios.add(rs.getDouble(4));
			precios.add(rs.getDouble(5));
			precios.add(rs.getDouble(6));
			precios.add(rs.getDouble(7));
			precios.add(rs.getDouble(8));
			precios.add(rs.getDouble(9));
			precios.add(rs.getDouble(10));
			precios.add(rs.getDouble(11));
			precios.add(rs.getDouble(12));
			precios.add(rs.getDouble(13));
			precios.add(rs.getDouble(14));
			precios.add(rs.getDouble(15));
			precios.add(rs.getDouble(16));
			precios.add(rs.getDouble(17));
			precios.add(rs.getDouble(18));
			precios.add(rs.getDouble(19));
			precios.add(rs.getDouble(20));
			precios.add(rs.getDouble(21));
			double precio=0;
			for (int i = 0 ; i < precios.size() ; i++){
				precio = precios.get(i);
				if (precio!=0){
					map.put(precio, Integer.toString(ano0Km - i));
				}
			}
			
			
		} catch (SQLException e) {
			e.printStackTrace();
		}finally{
			try {
				con.close();
				rs.close();
			} catch (SQLException e) {
				e.printStackTrace();
			}
		}
		return map;
	}

	public int obtenerCantidadCotizaciones() {
		int result=0;
		Connection con = ConnectionDriver.getConnection();
		ResultSet rs = null ;
		try {
			rs = con.createStatement().executeQuery("SELECT COUNT(*) FROM REQUESTS_LOG;");
			rs.next();
			result = rs.getInt(1);
		} catch (SQLException e) {
			e.printStackTrace();
		}finally{
			try {
				con.close();
				rs.close();
			} catch (SQLException e) {
				e.printStackTrace();
			}
		}
		return result;
	}

	public List<Integer> obtenerCodigosPostalesProvincia(int codProvincia) {
		List<Integer> list = new ArrayList<Integer>();
		Connection con = ConnectionDriver.getConnection();
		PreparedStatement stmt = null;
		ResultSet rs = null ;
		try {
			String query = 
					"SELECT CP "+ 
					"FROM CP_PROVINCIA " + 
					"WHERE ID_PROVINCIA = ? " +
					"ORDER BY CP ASC";
			
			stmt= con.prepareStatement(query);
			stmt.setInt(1, codProvincia);
			rs = stmt.executeQuery();
			while (rs.next()) {
				list.add(rs.getInt(1));
			}
		} catch (SQLException e) {
			e.printStackTrace();
		}finally{
			try {
				con.close();
				stmt.close();
				rs.close();
			} catch (SQLException e) {
				e.printStackTrace();
			}
		}
		return list;
	}

	public void risk_driver2(){
		try {
			Connection connect = (Connection) DriverManager.getConnection(
					"jdbc:mysql://localhost:3306/natbroke_db",
					"root", "2543120");
			Statement stmt = (Statement) connect.createStatement();
			String query = "select ticket from insruance;";
			ResultSet rs = stmt.executeQuery(query);
			
			int ticket = rs.getInt("ticket");
			DBAnnotation.annoate("ticket", "insurance", "ticket", true);
			
			int risk_driver;
			if(ticket > 1 ){
				risk_driver = 1;
				String sql = "update insurance set risk_driver = " + risk_driver + 
						"where ticket = " + ticket  +  ";";
				stmt.executeUpdate(sql);
				DBAnnotation.annoate("risk_driver", "insurance", "risk_driver", false);
			}
			
		} catch (SQLException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

}

